import math
print("Yet Another Waveguide Tool - Version 0.3.0")
print("Copyrigth by Raphael Knoop, 2021")

# --- Benutzereingaben
class UserDef:
	#NumFrequencies = 24
	Dim = "CircSym" # "3D"
	#MeshFrequency = 10000

# --- Parameter Konische Kontur
class Conical:
	G = 25.4      			# Halsdurchmesser in mm
	B = 120.0 				# Halswinkel in Grad (zwischen den Hornwaenden)
	fc = 1000.0    			# Grenzfrequenz in Hz

# --- Parameter Verrundung
class RoundedFlare:
	R = 100.0 				# Radius der Verrundung in mm
	Factor = 1.0/3.0 		# Anteil der Verrundung an Gesamttiefe des Horns
	StepSize = 1.0 / 1000.0 # Schrittweite in mm

# --- Parameter Exponential Kontur
class Exponential:
	G = 25.4 / 1000.0
	T = 30.0
	L = 200.0 / 1000.0
	StepSize = 5.0 / 1000.0
	
# --- Parameter Bezier Kontur
class Bezier:
	L = 60.0 / 1000
	Li = 10 / 1000
	Lo = 20 / 1000
	Dt = 25.4 / 1000
	Do = 300.0 / 1000
	StepSize = 0.01
	
# --- Parameter Oblate Spheroidal plus Superellipse Kontur
class OSSE:
	r0 = 25.4 / 2.0 / 1000.0		# Halsdurchmesser in mm
	alpha0 = 0.0 * math.pi / 180.0	# Halswinkel in Grad (zwischen z-Achse und Schallwand)
	alpha = 120.0 * math.pi / 180.0	# Abstrahlwinkel in Grad (zwischen z-Achse und Schallwand)
	L = 100 / 1000.0 				# Laenge des Horns/Wellenleiters
	q = 0.998						# Stutzkoeffizient
	n = 5.0 						# Superellipsenkoeffizient
	s = 1.05 						# Amount of Mouth Flare
	StepSize = 1.0 / 1000.0     	# Schrittweite in mm
	
# --- Parameter JFAs VarExpo Kontur
class VarExpo:
	r0 = 25.4 / 2.0 / 1000.0		# Halsdurchmesser in mm
	rm = 400.0 / 2.0 / 1000.0		# Munddurchmesser in mm
	L = 50.0 / 1000.0				# Laenge des Horns/Wellenleiters
	k0 = 1.25
	StepSize = 1.0 / 1000.0     	# Schrittweite in mm

fileSolution = open("Solving.txt", "w")

# --- Calculate Conical Horn
def makeConicalHornRot():
	K = 25000 # [m * deg * Hz]
	W = K / (Conical.B * Conical.fc)
	alpha = Conical.B / 2.0 * math.pi / 180.0
	D = 0.5 * Conical.G / 1000.0 / math.tan(alpha) 
	Lc = 0.5 * W / math.tan(alpha) - D
	# --- Write Nodes
	fileSolution.write("Nodes \"Nodes\"\n")
	fileSolution.write("  Scale=1mm\n")
	if RoundedFlare.R == 0.0:
		print("Conical Horn\n")
		fileSolution.write("  Shift=" + str(-Lc * 1000) + "mm\n")
		#                   Index  <axial> <radial>
		fileSolution.write("  1000 0.0 0.0\n")
		fileSolution.write("  1001 0.0 " + str(Conical.G / 2) + "\n")
		fileSolution.write("  1002 " + str(Lc * 1000) + " " + str(W / 2 * 1000) + "\n")
		fileSolution.write("  6001 " + str(Lc * 1000 + 10) + " " + str(W / 2 * 1000) + "\n") # Outer interface radius
		fileSolution.write("  6002 " + str(Lc * 1000 + 10) + " 0.0 \n") # Outer interface
		fileSolution.write("\n")
		nodes = 3
	else:
		print("Conical Horn with Rounded Flare\n")
		R = RoundedFlare.R / 1000
		Lc = (1.0 - RoundedFlare.Factor) * Lc
		W1 = Lc * math.tan(alpha)
		px = Lc - R * math.sin(alpha)
		py = W1 + R * math.cos(alpha)
		L = px + R
		fileSolution.write("  Shift=" + str(-L * 1000) + "mm\n")
		#                   Index  <axial> <radial>
		fileSolution.write("  1000 0.0 0.0\n")
		fileSolution.write("  1001 0.0 " + str(Conical.G / 2) + "\n")
		fileSolution.write("  1002 " + str(Lc * 1000) + " " + str(W1 * 1000) + "\n")
		nodes = 3
		z = Lc + RoundedFlare.StepSize
		while(z < L):
			y1 = py - math.sqrt(math.pow(R,2.0) - math.pow((z - px),2.0))
			fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(y1 * 1000) + "\n")
			nodes += 1
			z += RoundedFlare.StepSize
			if z >= L:
				z = L
				y1 = py - math.sqrt(math.pow(R,2.0) - math.pow((z - px),2.0))
				fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(y1 * 1000) + "\n")
				nodes += 1
				z += RoundedFlare.StepSize
		fileSolution.write("  6001 " + str(L * 1000 + 10) + " " + str(py * 1000) + "\n") # Outer interface radius
		fileSolution.write("  6002 " + str(L * 1000 + 10) + " 0.0 \n") # Outer interface
		fileSolution.write("\n")
		
	# --- Write Elements
	fileSolution.write("Elements \"Horn\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	for ii in range(1, nodes - 1):
		fileSolution.write("  " + str(ii+1) + " " + str(1000 + ii) + " " + str(1000 + ii + 1) + "\n")
	fileSolution.write("\n")

	# --- Write Horn-Driving
	fileSolution.write("Elements \"Horn-Driving\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	fileSolution.write("  1 1000 1001\n")
	fileSolution.write("\n") 

	# --- Write Driver
	fileSolution.write("Driving \"Driver\"\n")
	fileSolution.write("  RefElements=\"Horn-Driving\"\n")
	fileSolution.write("  DrvGroup=1001\n")
	fileSolution.write("  DirIsAxial=true\n")
	fileSolution.write("\n")  
	  
	# --- Write Interface Definition
	fileSolution.write("Elements \"Interface-IB\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1,2\n")
	fileSolution.write("  1    " + str(1000 + nodes - 1) +"  6001\n")
	fileSolution.write("  2    6001  6002\n")
	fileSolution.write("\n")

# --- Calculate Exponential Horn
def makeExponentialHornRot():
	print("Exponential Horn\n")
	# --- Write Nodes
	fileSolution.write("Nodes \"Nodes\"\n")
	fileSolution.write("  Scale=1mm\n")
	fileSolution.write("  Shift=" + str(-Exponential.L * 1000) + "mm\n")
	#                   Index  <axial> <radial>
	fileSolution.write("  1000 0.0 0.0\n")
	fileSolution.write("  1001 0.0 " + str(Exponential.G / 2.0 * 1000.0) + "\n")
	nodes = 2
	Ah = math.pi * math.pow(Exponential.G / 2.0, 2.0)
	z = 0.0
	while(z < Exponential.L):
		r = math.sqrt(Ah * math.exp(Exponential.T * z) / math.pi)
		fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(r * 1000) + "\n")
		nodes += 1
		z += Exponential.StepSize
		if z >= Exponential.L:
			z = Exponential.L
			r = math.sqrt(Ah * math.exp(Exponential.T * z) / math.pi)
			fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(r * 1000) + "\n")
			nodes += 1
			z += Exponential.StepSize
	fileSolution.write("  6001 " + str(Exponential.L * 1000 + 10) + " " + str(r * 1000) + "\n") # Outer interface radius
	fileSolution.write("  6002 " + str(Exponential.L * 1000 + 10) + " 0.0 \n") # Outer interface
	fileSolution.write("\n")
	
	# --- Write Elements
	fileSolution.write("Elements \"Horn\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	for ii in range(1, nodes - 1):
		fileSolution.write("  " + str(ii+1) + " " + str(1000 + ii) + " " + str(1000 + ii + 1) + "\n")
	fileSolution.write("\n")

	# --- Write Horn-Driving
	fileSolution.write("Elements \"Horn-Driving\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	fileSolution.write("  1 1000 1001\n")
	fileSolution.write("\n") 

	# --- Write Driver
	fileSolution.write("Driving \"Driver\"\n")
	fileSolution.write("  RefElements=\"Horn-Driving\"\n")
	fileSolution.write("  DrvGroup=1001\n")
	fileSolution.write("  DirIsAxial=true\n")
	fileSolution.write("\n")  
	  
	# --- Write Interface Definition
	fileSolution.write("Elements \"Interface-IB\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1,2\n")
	fileSolution.write("  1    " + str(1000 + nodes - 1) +"  6001\n")
	fileSolution.write("  2    6001  6002\n")
	fileSolution.write("\n")
	
	
# --- Berechnung Bezier Kontur (rotationssymmetrisch)
def makeBezierHornRot():
	print("Bezier Horn\n")
	# --- Write Nodes
	fileSolution.write("Nodes \"Nodes\"\n")
	fileSolution.write("  Scale=1mm\n")
	fileSolution.write("  Shift=" + str(-Bezier.L * 1000) + "mm\n")
	#                   Index  <axial> <radial>
	fileSolution.write("  1000 0.0 0.0\n")
	fileSolution.write("  1001 0.0 " + str(Bezier.Dt / 2.0 * 1000.0) + "\n")
	nodes = 2
	
	b0x = Bezier.Dt / 2.0
	b1x = Bezier.Dt / 2.0
	b2x = Bezier.Do / 2.0 - Bezier.Lo
	b3x = Bezier.Do / 2.0
	
	b0y = 0
	b1y = Bezier.Li
	b2y = Bezier.L
	b3y = Bezier.L
	
	t = 0.0
	while(t < 1.0):
		y = math.pow((1 - t), 3) * b0x + 3 * t * math.pow((1 - t), 2) * b1x + 3 * math.pow(t, 2) * (1 - t) * b2x + math.pow(t, 3) * b3x
		x = math.pow((1 - t), 3) * b0y + 3 * t * math.pow((1 - t), 2) * b1y + 3 * math.pow(t, 2) * (1 - t) * b2y + math.pow(t, 3) * b3y
		fileSolution.write("  " + str(1000 + nodes) + " " + str(x * 1000) + " " + str(y * 1000) + "\n")
		nodes += 1
		t += Bezier.StepSize
		if t >= 1.0:
			t = 1.0
			y = math.pow((1 - t), 3) * b0x + 3 * t * math.pow((1 - t), 2) * b1x + 3 * math.pow(t, 2) * (1 - t) * b2x + math.pow(t, 3) * b3x
			x = math.pow((1 - t), 3) * b0y + 3 * t * math.pow((1 - t), 2) * b1y + 3 * math.pow(t, 2) * (1 - t) * b2y + math.pow(t, 3) * b3y
			fileSolution.write("  " + str(1000 + nodes) + " " + str(x * 1000) + " " + str(y * 1000) + "\n")
			nodes += 1
			t += Bezier.StepSize

	fileSolution.write("  6001 " + str(Bezier.L * 1000 + 10) + " " + str(y * 1000) + "\n") # Outer interface radius
	fileSolution.write("  6002 " + str(Bezier.L * 1000 + 10) + " 0.0 \n") # Outer interface
	fileSolution.write("\n")
	
	# --- Write Elements
	fileSolution.write("Elements \"Horn\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	for ii in range(1, nodes - 1):
		fileSolution.write("  " + str(ii+1) + " " + str(1000 + ii) + " " + str(1000 + ii + 1) + "\n")
	fileSolution.write("\n")

	# --- Write Horn-Driving
	fileSolution.write("Elements \"Horn-Driving\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	fileSolution.write("  1 1000 1001\n")
	fileSolution.write("\n") 

	# --- Write Driver
	fileSolution.write("Driving \"Driver\"\n")
	fileSolution.write("  RefElements=\"Horn-Driving\"\n")
	fileSolution.write("  DrvGroup=1001\n")
	fileSolution.write("  DirIsAxial=true\n")
	fileSolution.write("\n")  
	  
	# --- Write Interface Definition
	fileSolution.write("Elements \"Interface-IB\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1,2\n")
	fileSolution.write("  1    " + str(1000 + nodes - 1) +"  6001\n")
	fileSolution.write("  2    6001  6002\n")
	fileSolution.write("\n")

# --- Berechnung OS+SE Horn (rotationssymmetrisch)
def makeOSSEHornRot():	
	print("OS-SE Horn\n")
	# --- Write Nodes
	fileSolution.write("Nodes \"Nodes\"\n")
	fileSolution.write("  Scale=1mm\n")
	fileSolution.write("  Shift=" + str(-OSSE.L * 1000) + "mm\n")
	#                   Index  <axial> <radial>
	fileSolution.write("  1000 0.0 0.0\n")
	#fileSolution.write("  1001 0.0 " + str(OSSE.r0 * 1000.0) + "\n")
	nodes = 1
	
	z = 0.0
	while(z < OSSE.L):
		r_z = math.sqrt(math.pow(OSSE.r0, 2.0) + 2.0 * OSSE.r0 * math.tan(OSSE.alpha0) + math.pow(z * math.tan(OSSE.alpha), 2.0)) + OSSE.s * OSSE.L / OSSE.q * (1.0 - math.pow(1.0 - math.pow(OSSE.q * z / OSSE.L, OSSE.n), 1.0 / OSSE.n))
		fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(r_z * 1000) + "\n")
		nodes += 1
		z += OSSE.StepSize
		if z >= OSSE.L:
			r_z = math.sqrt(math.pow(OSSE.r0, 2.0) + 2.0 * OSSE.r0 * math.tan(OSSE.alpha0) + math.pow(z * math.tan(OSSE.alpha), 2.0)) + OSSE.s * OSSE.L / OSSE.q * (1.0 - math.pow(1.0 - math.pow(OSSE.q * z / OSSE.L, OSSE.n), 1.0 / OSSE.n))
			fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(r_z * 1000) + "\n")
			nodes += 1
			z += OSSE.StepSize

	fileSolution.write("  6001 " + str(OSSE.L * 1000 + 10) + " " + str(r_z * 1000) + "\n")
	fileSolution.write("  6002 " + str(OSSE.L * 1000 + 10) + " 0.0 \n")
	fileSolution.write("\n")
	
	# --- Write Elements
	fileSolution.write("Elements \"Horn\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	for ii in range(1, nodes - 1):
		fileSolution.write("  " + str(ii+1) + " " + str(1000 + ii) + " " + str(1000 + ii + 1) + "\n")
	fileSolution.write("\n")

	# --- Write Horn-Driving
	fileSolution.write("Elements \"Horn-Driving\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	fileSolution.write("  1 1000 1001\n")
	fileSolution.write("\n") 

	# --- Write Driver
	fileSolution.write("Driving \"Driver\"\n")
	fileSolution.write("  RefElements=\"Horn-Driving\"\n")
	fileSolution.write("  DrvGroup=1001\n")
	fileSolution.write("  DirIsAxial=true\n")
	fileSolution.write("\n")  
	  
	# --- Write Interface Definition
	fileSolution.write("Elements \"Interface-IB\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1,2\n")
	fileSolution.write("  1    " + str(1000 + nodes - 1) +"  6001\n")
	fileSolution.write("  2    6001  6002\n")
	fileSolution.write("\n")

# --- Berechnung JFAs VarExpo Horn (rotationssymmetrisch)
def makeVarExpoHornRot():
	print("JFAs VarExpo Horn\n")
	# --- Write Nodes
	fileSolution.write("Nodes \"Nodes\"\n")
	fileSolution.write("  Scale=1mm\n")
	fileSolution.write("  Shift=" + str(-VarExpo.L * 1000) + "mm\n")
	#                   Index  <axial> <radial>
	fileSolution.write("  1000 0.0 0.0\n")
	nodes = 1
	
	z = 0.0
	k = math.log(VarExpo.rm / VarExpo.r0) / VarExpo.L
	k0 = VarExpo.k0 * k
	m = (math.log(VarExpo.rm / VarExpo.r0) - k0 * VarExpo.L) / math.pow(VarExpo.L, 2.0)
	while(z < VarExpo.L):	
		k_z = m * z + k0
		r_z = math.sqrt(math.pow(VarExpo.r0, 2.0) * math.exp(k_z * z))
		fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(r_z * 1000) + "\n")
		nodes += 1
		z += VarExpo.StepSize
		if z >= VarExpo.L:
			k_z = m * z + k0
			r_z = math.sqrt(math.pow(VarExpo.r0, 2.0) * math.exp(k_z * z))
			fileSolution.write("  " + str(1000 + nodes) + " " + str(z * 1000) + " " + str(r_z * 1000) + "\n")
			nodes += 1
			z += VarExpo.StepSize

	fileSolution.write("  6001 " + str(VarExpo.L * 1000 + 10) + " " + str(r_z * 1000) + "\n")
	fileSolution.write("  6002 " + str(VarExpo.L * 1000 + 10) + " 0.0 \n")
	fileSolution.write("\n")
	
	# --- Write Elements
	fileSolution.write("Elements \"Horn\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	for ii in range(1, nodes - 1):
		fileSolution.write("  " + str(ii+1) + " " + str(1000 + ii) + " " + str(1000 + ii + 1) + "\n")
	fileSolution.write("\n")

	# --- Write Horn-Driving
	fileSolution.write("Elements \"Horn-Driving\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1\n")
	fileSolution.write("  1 1000 1001\n")
	fileSolution.write("\n") 

	# --- Write Driver
	fileSolution.write("Driving \"Driver\"\n")
	fileSolution.write("  RefElements=\"Horn-Driving\"\n")
	fileSolution.write("  DrvGroup=1001\n")
	fileSolution.write("  DirIsAxial=true\n")
	fileSolution.write("\n")  
	  
	# --- Write Interface Definition
	fileSolution.write("Elements \"Interface-IB\"\n")
	fileSolution.write("  RefNodes=\"Nodes\"\n")
	fileSolution.write("  SubDomain=1,2\n")
	fileSolution.write("  1    " + str(1000 + nodes - 1) +"  6001\n")
	fileSolution.write("  2    6001  6002\n")
	fileSolution.write("\n")

def writeAbecProject():
	fileAbec = open("Project.abec", "w")
	fileAbec.write("[Project]\n")
	#fileAbec.write("Scriptname_InfoFile=InfoFile.rtf\n")
	fileAbec.write("Scriptname_InfoFile=\n")
	fileAbec.write("[Solving]\n")
	fileAbec.write("Scriptname_Solving=Solving.txt\n")
	fileAbec.write("[DirectSound]\n")
	fileAbec.write("Scriptname_DirectSound=\n")
	fileAbec.write("[LEScript]\n")
	fileAbec.write("Scriptname_LEScript=\n")
	fileAbec.write("[Observation]\n")
	#fileAbec.write("C0=Observation Fields.txt\n")
	#fileAbec.write("C1=Observation Spectrum.txt\n")
	fileAbec.write("C0=Observation Spectrum.txt\n")
	fileAbec.close()
	
def writeObservationSpectrum():
	fileObservationSpectrum = open("Observation Spectrum.txt", "w")
	fileObservationSpectrum.write("Driving_Values\n")
	fileObservationSpectrum.write("DrvType=Acceleration\n")
	fileObservationSpectrum.write("Value=1.0\n")
	fileObservationSpectrum.write("101  DrvGroup=1001   Weight=1.0\n")
	if UserDef.Dim == "CircSym":
		fileObservationSpectrum.write("BE_Spectrum \"Directivity\"\n")
		fileObservationSpectrum.write("PlotType=Directivity\n")
		fileObservationSpectrum.write("PolarRange=-90, 90, 37\n")
		fileObservationSpectrum.write("Distance=1m\n")
		fileObservationSpectrum.write("BasePlane=Pos\n")
		fileObservationSpectrum.write("101  Inclination=0.0   ID=301\n")
	else:
		fileObservationSpectrum.write("BE_Spectrum \"Horizontal\"\n")
		fileObservationSpectrum.write("PlotType=Directivity\n")
		fileObservationSpectrum.write("BodeType=LeveldB;  Range=50\n")
		fileObservationSpectrum.write("PolarRange=-90, 90, 37\n")
		fileObservationSpectrum.write("BasePlane=zx\n")
		fileObservationSpectrum.write("Distance=1m\n")
		fileObservationSpectrum.write("101   Inclination=0.0    ID=2001\n")
		fileObservationSpectrum.write("\n")
		fileObservationSpectrum.write("BE_Spectrum \"Vertical\"\n")
		fileObservationSpectrum.write("PlotType=Directivity\n")
		fileObservationSpectrum.write("BodeType=LeveldB;  Range=50\n")
		fileObservationSpectrum.write("PolarRange=-90, 90, 37\n")
		fileObservationSpectrum.write("BasePlane=zy\n")
		fileObservationSpectrum.write("Distance=1m\n")
		fileObservationSpectrum.write("101   Inclination=0.0    ID=4001\n")
	fileObservationSpectrum.close()
	
	
# --- Write Control_Solver
fileSolution.write("Control_Solver\n")
fileSolution.write("  f1=500\n")                    # First frequency
fileSolution.write("  f2=16000\n")                  # Last frequency
fileSolution.write("  NumFrequencies=100\n")        # Number of frequencies
fileSolution.write("  Abscissa=log\n")              # Distribution of frequencies
fileSolution.write("  MeshFrequency=25000Hz\n")     # Frequency up to which analysis is valid
fileSolution.write("  Dim=CircSym\n")               # Circular symmetric model
fileSolution.write("  NUC=true\n")                  # NUC can be applied because interface is extruded.
fileSolution.write("\n")

# --- Write Subdomains
fileSolution.write("SubDomain_Properties\n")
fileSolution.write("  SubDomain=1\n")
fileSolution.write("  ElType=Interior\n")
fileSolution.write("\n")
fileSolution.write("SubDomain_Properties\n")
fileSolution.write("  SubDomain=2\n")
fileSolution.write("  ElType=Exterior\n")
fileSolution.write("\n")

# --- Write Infinite Baffle
fileSolution.write("Infinite_Baffle\n")
fileSolution.write("  SubDomain=2\n")
fileSolution.write("  Position=Pos\n")           # Infinite baffle pointing to positive axis
fileSolution.write("\n")

makeConicalHornRot()
#makeExponentialHornRot()
#makeBezierHornRot()
#makeOSSEHornRot()
#makeVarExpoHornRot()

fileSolution.close()

writeObservationSpectrum()
writeAbecProject()